@@ -1,3 +1,6 @@
+
+
+#include "ustring.h"
 #include "thot_gui.h"
 #include "thot_sys.h"
 #include "tree.h"
@@ -6,6 +9,7 @@
 #include "view.h"
 #include "selection.h"
 #include "conststr.h"
+#include "constint.h"
 #include "typestr.h"
 #define THOT_EXPORT 
 #include "constres.h"
@@ -31,23 +35,19 @@ char IndexSymols[] = {'%', /* pre couplage */
 		      'E', /* extension */
 		      '@', /* recursive */
 		      '*', /* source root (fictive) */
-		      '\0'};
+		      EOS};
 
 static StrUnitDesc *ListUnitDesc = NULL;
 static TypeTree BuildStack[MAXDEPTH];
 static int BuildStackTop;
+
+
 /*----------------------------------------------------------------------
   RestCompSymbols
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static int RestCompSymbols(char s1, char s2)
-#else  /* __STDC__ */
-static int RestCompSymbols(char s1, char s2)
-char s1;
-char s2;
-#endif  /* __STDC__ */
 {
-  boolean found = FALSE;
+  ThotBool found = FALSE;
   int i = 0;
 
   if (s1 == s2)
@@ -71,23 +71,37 @@ char s2;
   retourne 1 si first est avant second, 0 s'il est apres 2 si les deux 
   sont egaux.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static int RestIsBefore (TypeTree first, TypeTree second) 
-#else  /* __STDC__ */
-static int RestIsBefore (first, second) 
-TypeTree first;
-TypeTree second;
-#endif  /* __STDC__ */
 {
   TypeTree fChild, sChild;
+  int fDepth = 0;
+  int sDepth = 0; 
 
   fChild = RestNextEffNode (first, NULL);
+  if (fChild != NULL)
+    {
+      fDepth = fChild->TDepth;
+      if (fChild->TPrintSymb == '@')
+	{
+	  fChild = fChild->TRecursive;
+	  fDepth = 1000;
+	}
+    }
   sChild = RestNextEffNode (second, NULL);
+  if (sChild != NULL)
+    {
+      sDepth = sChild->TDepth;
+      if (sChild->TPrintSymb == '@')
+	{
+	  sChild = sChild->TRecursive;
+	  sDepth = 1000;
+	}
+    }
   if (fChild != NULL && sChild != NULL)
     {
-      if (fChild->TDepth > sChild->TDepth)
+      if (fDepth > sDepth)
 	return 0;
-      if (fChild->TDepth < sChild->TDepth)
+      if (fDepth < sDepth)
 	return 1;
     }
   while (fChild!=NULL && sChild != NULL && 
@@ -110,16 +124,10 @@ TypeTree second;
   RestInsertNode
   insere le noeud child dans l'arbre de types comme fils de parent
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void RestInsertNode (TypeTree child, TypeTree parent) 
-#else  /* __STDC__ */
-void RestInsertNode (child, parent) 
-TypeTree child;
-TypeTree parent;
-#endif  /* __STDC__ */
 {
   TypeTree sibling, last;
-  boolean found;
+  ThotBool found;
   
   last = NULL;
   sibling = parent->TChild;
@@ -134,7 +142,10 @@ TypeTree parent;
       found = FALSE;
       while (!found && sibling != NULL)
 	{
-	  found = (RestIsBefore (child, sibling) == 1);
+	  if (parent->TPrintSymb != '{')
+	    found = (RestIsBefore (child, sibling) == 1);
+	  else
+	    found = FALSE;
 	  if (!found)
 	    {
 	      last = sibling;
@@ -160,16 +171,13 @@ TypeTree parent;
   RestReorderChild
   detache et re-insere le noeud child dans l'arbre de types 
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void RestReorderChild (TypeTree child) 
-#else  /* __STDC__ */
-void RestReorderChild (child) 
-TypeTree child;
-#endif  /* __STDC__ */
 {
   TypeTree parent;
   
-  if (child != NULL && child->TParent != NULL)
+  if (child != NULL && 
+      child->TParent != NULL && 
+      child->TParent->TPrintSymb != '{')
     {
       parent = child->TParent;
       if (child->TPrevious == NULL)
@@ -185,17 +193,14 @@ TypeTree child;
   RestNewTypeNode
   alloue un noeud d'arbre de types.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static TypeTree RestNewTypeNode(ElementType elemType)
-#else  /* __STDC__ */
-static TypeTree RestNewTypeNode(elemType)
-ElementType elemType;
-#endif  /* __STDC__ */
 {
   TypeTree theTree = NULL;
+  int i;
 
   theTree = (TypeTree)TtaGetMemory(sizeof(StrTypeTree));
   theTree->TId = (RContext->CIdCount)++;
+  theTree->TSchema = elemType.ElSSchema;
   theTree->TParent = NULL;
   theTree->TChild = NULL;
   theTree->TPrevious = NULL;
@@ -203,22 +208,22 @@ ElementType elemType;
   theTree->TRecursive = NULL;
   theTree->TRecursApplied = FALSE;
   theTree->TEffective = TRUE;
-  theTree->TPrintSymb = ' ';
+  theTree->TDiscard = FALSE;
+  theTree->TPrintSymb = SPACE;
   theTree->TypeNum = elemType.ElTypeNum;
   theTree->TDepth = 0;
+  for (i = 0; i < MAXNODECOUPLE; i++)
+    {
+      theTree->TPreCouple[i].SourceContext = NULL;
+      theTree->TPreCouple[i].DestContext = NULL;
+    }
   return theTree;
 }
 /*----------------------------------------------------------------------
   RestSearchPreCouple
   recherche si typeNode est compatible avec un prcouplage
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void RestSearchPreCouple (TypeTree typeNode, boolean isSource) 
-#else  /* __STDC__ */
-static void RestSearchPreCouple (typeNode, isSource) 
-TypeTree typeNode;
-boolean isSource;
-#endif  /* __STDC__ */
+static void RestSearchPreCouple (TypeTree typeNode, ThotBool isSource) 
 {
   
 }
@@ -227,17 +232,11 @@ boolean isSource;
   RecBuildTypeTree
   construit l'arbre des types du type elemType
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static TypeTree RecBuildTypeTree (ElementType elemType, boolean isSource) 
-#else  /* __STDC__ */
-static TypeTree RecBuildTypeTree (elemType, isSource) 
-ElementType elemType;
-boolean isSource;
-#endif  /* __STDC__ */
+static TypeTree RecBuildTypeTree (ElementType elemType, ThotBool isSource) 
 {
   TypeTree theTree = NULL;
   TypeTree newNode;
-  SRule *strRule;
+  PtrSRule strRule;
   ElementType childType;
   int i;
 
@@ -248,7 +247,7 @@ boolean isSource;
       for (i = BuildStackTop; i >= 0 && theTree == NULL; i--)
 	{
 	  newNode = BuildStack[i];
-	  if(newNode->TypeNum == elemType.ElTypeNum)
+	  if(newNode->TypeNum == elemType.ElTypeNum && newNode->TSchema == elemType.ElSSchema)
 	    {
 	      theTree = RestNewTypeNode(elemType);
 	      theTree->TRecursive = newNode;
@@ -259,7 +258,7 @@ boolean isSource;
       /* construit un noeud */
       if (theTree == NULL)
 	{
-	  strRule = &(((PtrSSchema)(elemType.ElSSchema))->SsRule[(elemType.ElTypeNum) - 1]);
+	  strRule = ((PtrSSchema)(elemType.ElSSchema))->SsRule->SrElem[(elemType.ElTypeNum) - 1];
 	  childType.ElSSchema = elemType.ElSSchema;
 	  if (strRule->SrConstruct == CsIdentity)
 	    {
@@ -268,6 +267,20 @@ boolean isSource;
 	      theTree = RecBuildTypeTree (childType, isSource);
 	      theTree->TypeNum = elemType.ElTypeNum;
 	    }
+	  else if (strRule->SrConstruct == CsNatureSchema && strRule->SrSSchemaNat != NULL)
+	    { /* schema externe explicite */
+	      childType.ElSSchema = (SSchema) strRule->SrSSchemaNat;
+	      if (childType.ElSSchema == NULL)
+		{
+#ifdef DEBUG
+		  fprintf (stderr, "Charge nature %s\n", strRule->SrOrigNat);
+#endif
+		  childType.ElSSchema = TtaNewNature (0, elemType.ElSSchema, NULL, strRule->SrOrigNat, "");
+		}
+	      childType.ElTypeNum  = ((PtrSSchema)(childType.ElSSchema))->SsRootElem;
+	      theTree = RecBuildTypeTree (childType, isSource);
+	      theTree->TypeNum = childType.ElTypeNum;
+	    }
 	  else
 	    {
 	      /* creation d'un noeud */
@@ -298,7 +311,7 @@ boolean isSource;
 		      theTree->TPrintSymb = 'U';
 		    }
 		  else if (strRule->SrNChoices == -1)
-		    {
+		    { /* it is an external schema*/
 		      theTree->TPrintSymb = 'N';
 		    }
 		  break;
@@ -330,15 +343,16 @@ boolean isSource;
 		    case 2:
 		      theTree->TPrintSymb = 'G';
 		      break;
-		    case 4:
+		    case 3:
 		      theTree->TPrintSymb = 'S';
 		      break;
-		    case 5:
-		      theTree->TPrintSymb = 'R';
-		      break;
-		    case 6:
+		    case 4:
 		      theTree->TPrintSymb = 'P';
 		      break;
+/*****		    case 6:
+		      theTree->TPrintSymb = 'P';
+		      break;
+******/
 		    }
 		  break;
 		case CsNatureSchema:
@@ -365,46 +379,13 @@ boolean isSource;
   retourne un pointeur sur le noeud de l'arbre de type du schema 
   correspondant au type d'element elemType 
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static TypeTree GetTypeTree(ElementType elemType, boolean isSource)
-#else  /* __STDC__ */
-static TypeTree GetTypeTree(elemType, isSource)
-ElementType elemType;
-boolean isSource;
-#endif  /* __STDC__ */
+static TypeTree GetTypeTree(ElementType elemType, ThotBool isSource)
 {
   BuildStackTop = -1; 
   return RecBuildTypeTree (elemType, isSource); 
 }
 
 
-/*----------------------------------------------------------------------
-  FindTypeInTree
-  retourne un pointeur sur le noeud de type typeNum de l'arbre de types 
-  typeTree - profondeur d'abord
-  ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static TypeTree FindTypeInTree (int typeNum, TypeTree tree)
-#else  /* __STDC__ */
-static TypeTree FindTypeInTree (typeNum, tree)
-int typeNum;
-TypeTree tree;
-#endif  /* __STDC__ */
-
-{
-  TypeTree child, res;
-  
-  if (tree == NULL || tree->TypeNum == typeNum)
-    return tree;
-  child = tree->TChild;
-  res = NULL;
-  while (res == NULL && child != NULL)
-    {
-      res = FindTypeInTree (typeNum, child);
-      child = child->TNext;
-    }
-  return res;
-}
 
 
 /*----------------------------------------------------------------------  
@@ -427,9 +408,11 @@ static TypeTree GetElemSourceTree()
   theTree->TRecursive = NULL;
   theTree->TRecursApplied = FALSE;
   theTree->TEffective = FALSE;
+  theTree->TDiscard = FALSE;
   theTree->TPrintSymb = '*';
   theTree->TId = (RContext->CIdCount)++;
   theTree->TypeNum = 0;
+  theTree->TSchema = NULL;
   theTree->TDepth = 0;
   nbTypes = 0;
   
@@ -454,19 +437,14 @@ static TypeTree GetElemSourceTree()
     }
   else
     return theTree; 
-}
+} 
 
 
 /*----------------------------------------------------------------------  
   RestFreeTree
   libere un arbre de types
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void RestFreeTree(TypeTree tree)
-#else  /* __STDC__ */
-void RestFreeTree(tree)
-TypeTree tree;
-#endif  /* __STDC__ */
 {
   TypeTree child, nextchild;
   if (tree != NULL)
@@ -478,7 +456,7 @@ TypeTree tree;
           RestFreeTree(child);
           child = nextchild;
         }
-      TtaFreeMemory((char *)tree);
+      TtaFreeMemory (tree);
     }
 }
 /*----------------------------------------------------------------------  
@@ -486,12 +464,7 @@ TypeTree tree;
   retourne le symbole correspondant au noeud unite tree ('U' si aucune
   information n'est presente pour ce noeud).
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static char GetUDescSymb (TypeTree tree)
-#else  /* __STDC__ */
-static char GetUDescSymb (tree)
-TypeTree tree;
-#endif  /* __STDC__ */	
 {
   StrUnitDesc *ptrDesc;
 
@@ -501,7 +474,7 @@ TypeTree tree;
   if (ptrDesc == NULL)
     return 'U';
   else
-    return IndexSymols[ptrDesc->TypeNumber - 1];
+    return IndexSymols[ptrDesc->TypeNumber];
 }
 
 /*----------------------------------------------------------------------  
@@ -511,13 +484,7 @@ TypeTree tree;
   retire l'entree si elle existe deja et n'est pas instanciee avec le
   type de base donne
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void AddUnitDescriptor (TypeTree tree, ElementType elType)
-#else  /* __STDC__ */
-static void AddUnitDescriptor (tree, elType)
-TypeTree tree;
-ElementType elType;
-#endif  /* __STDC__ */
 {
   StrUnitDesc *ptrDesc, *previous;
   
@@ -544,23 +511,15 @@ ElementType elType;
 	  previous->Next =  ptrDesc->Next;
 	else
 	  ListUnitDesc = ptrDesc->Next;
-	TtaFreeMemory ((char *)ptrDesc);
+	TtaFreeMemory (ptrDesc);
       }
 }
 
 /*----------------------------------------------------------------------  
   RestRecPrintType
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void RestRecPrintType(TypeTree theTree, char *buffer, TypeTree *nodes, int *indice, PrintMethod method)
-#else  /* __STDC__ */
-static void RestRecPrintType(theTree, buffer, nodes, indice, method)
-TypeTree theTree;
-char *buffer;
-TypeTree *nodes;
-int *indice;
-PrintMethod method;
-#endif  /* __STDC__ */
+static void RestRecPrintType(TypeTree theTree, char *buffer, TypeTree *nodes,
+			     int *indice, PrintMethod method)
 {
   TypeTree child, markedTree;
 
@@ -609,7 +568,7 @@ PrintMethod method;
 		{ 
 		  child = theTree->TChild;
 		  /* saute les noeuds non effectifs */
-		  /*while (effect && child != NULL && !child->TEffective) 
+		  /* while (effect && child != NULL && !child->TEffective) 
 		    child = child->Next;*/
 		}
 	    }
@@ -652,14 +611,7 @@ PrintMethod method;
   empreinte effective
   si effective est vrai cree l'empreinte des types effectifs 
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void RestPrintType(void *restruct, int bufsize, boolean isSource)
-#else  /* __STDC__ */
-void RestPrintType(restruct, bufsize, isSource)
-void *restruct;
-int bufsize;
-boolean isSource;
-#endif  /* __STDC__ */
+void RestPrintType(void *restruct, int bufsize, ThotBool isSource)
 {
   int indice;
   TypeTree childTree;
@@ -689,7 +641,7 @@ boolean isSource;
 			  srcInfo->SNodes, 
 			  &indice, 
 			  srcInfo->SPrintMethod);
-      srcInfo->SPrint[indice] = '\0';
+      srcInfo->SPrint[indice] = EOS;
     }
   else
     {
@@ -699,7 +651,7 @@ boolean isSource;
 			dstInfo->RDestNodes,
 			&indice,
 			PM_GENERIC);
-      dstInfo->RDestPrint[indice] = '\0';
+      dstInfo->RDestPrint[indice] = EOS;
     }
 }     
   
@@ -707,15 +659,10 @@ boolean isSource;
   RestCompatible
   renvoie vrai si elem est une instance de tree
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static boolean RestCompatible (ElementType elType, TypeTree tree)
-#else  /* __STDC__ */
-static boolean RestCompatible (elType, tree)
-ElementType elType;
-TypeTree tree;
-#endif  /* __STDC__ */
+static ThotBool RestCompatible (ElementType elType, TypeTree tree)
 {
-  boolean result = FALSE;
+  ThotBool result = FALSE;
+  ElementType elTTree;
 
   switch (tree->TPrintSymb)
     {
@@ -739,7 +686,11 @@ TypeTree tree;
 	    result = TRUE;
 	}
       else
-	result = (elType.ElTypeNum == tree->TypeNum);
+	{
+	  elTTree.ElSSchema = tree->TSchema;
+	  elTTree.ElTypeNum = tree->TypeNum;
+	  result = TtaSameTypes (elType, elTTree);
+	}
       break;
     }
   return result;
@@ -751,26 +702,21 @@ TypeTree tree;
   RecInstanciateType : marque les noeuds de tree comme effectifs s'il
   sonts representes dans l'instance elem.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static boolean RecInstanciateType(TypeTree tree, Element elem, PrintMethod method)
-#else  /* __STDC__ */
-static boolean RecInstanciateType(tree, elem, method)
-TypeTree tree;
-Element elem;
-PrintMethod method;
-#endif  /* __STDC__ */
+static ThotBool RecInstanciateType (TypeTree tree, Element elem, PrintMethod method)
 {
   ElementType elType, childType;
-  boolean result, found, alone;
-  Element elemChild = NULL;
-  TypeTree treeChild, treeChild2;
+  ThotBool    result, found, alone;
+  Element     elemChild = NULL;
+  TypeTree    treeChild, treeChild2;
 
   if (tree == NULL || elem == NULL)
     return FALSE;
   elType = TtaGetElementType (elem);
   if (!RestCompatible(elType, tree))
     return FALSE;
+
   result = FALSE;
+  alone = FALSE;
   if (TtaGetConstructOfType (elType) == ConstructIdentity 
       && tree->TChild == NULL 
       && tree->TPrintSymb != '@')
@@ -838,6 +784,7 @@ PrintMethod method;
 	  TtaNextSibling (&elemChild);
 	}
     }
+
   if (result && 
       !((method & PM_WTHTLIST) && tree->TPrintSymb == '(' && alone) &&
       !((method & PM_WTHTCHOICE) && tree->TPrintSymb == '[')) 
@@ -845,18 +792,16 @@ PrintMethod method;
     /* au moins un des fils est completement instancie' */
     /* (existence d'une feuille; on instancie elem */
     tree->TEffective = TRUE;
+  else if (result)
+    tree->TDiscard = TRUE;
+    
   return result;
 }
 
 /*----------------------------------------------------------------------  
   RestUnmarkEffective
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void RestUnmarkEffective (TypeTree tree)
-#else  /* __STDC__ */
-static void RestUnmarkEffective (tree)
-TypeTree tree;
-#endif  /* __STDC__ */
 {
   TypeTree child;
   StrUnitDesc *ptrDesc;
@@ -864,7 +809,10 @@ TypeTree tree;
   if (tree != NULL)
     {
       if (tree->TPrintSymb != '*')
-	tree->TEffective = FALSE;
+	{
+	  tree->TEffective = FALSE;
+	  tree->TDiscard = FALSE;
+	}
       child = tree->TChild;
       while (child != NULL)
 	{
@@ -876,7 +824,7 @@ TypeTree tree;
   while (ptrDesc != NULL)
     {
       ListUnitDesc = ptrDesc->Next;
-      TtaFreeMemory((char *)ptrDesc);
+      TtaFreeMemory (ptrDesc);
       ptrDesc = ListUnitDesc;
     }
   ListUnitDesc = NULL;
@@ -885,16 +833,9 @@ TypeTree tree;
 /*----------------------------------------------------------------------  
   RestInstanciateType : instanciation de l'arbre des types 
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static boolean RestInstanciateType(SourcePrint sourcePrint, PrintMethod method)
-#else  /* __STDC__ */
-static boolean RestInstanciateType(sourcePrint, method)
-Restruct restruct;
-Element elem;
-PrintMethod method;
-#endif  /* __STDC__ */
+static ThotBool RestInstanciateType(SourcePrint sourcePrint, PrintMethod method)
 {
-  boolean ok = TRUE;
+  ThotBool ok = TRUE;
   int i;
 
   RestUnmarkEffective (sourcePrint->STree);
@@ -912,21 +853,16 @@ PrintMethod method;
 /*----------------------------------------------------------------------  
   RestGetSourcePrint
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static SourcePrint RestGetSourcePrint (PrintMethod method)
-#else  /* __STDC__ */
-static SourcePrint RestGetSourcePrint (method)
-PrintMethod method;
-#endif  /* __STDC__ */
 {
   SourcePrint sPrint;
-  boolean found = FALSE;
+  ThotBool found = FALSE;
   int i;
 
   sPrint = RContext->CListSourcePrints;
   if (sPrint == NULL)
     {
-      RContext->CListSourcePrints = TtaGetMemory (sizeof(StrSourcePrint));
+      RContext->CListSourcePrints = (StrSourcePrint*)TtaGetMemory (sizeof(StrSourcePrint));
       sPrint = RContext->CListSourcePrints;
       sPrint->SNext = NULL;
       sPrint->STree = NULL;
@@ -937,9 +873,10 @@ PrintMethod method;
       {
 	found = (sPrint->SPrintMethod == method);
 	if (!found)
+	  {
 	  if (sPrint->SNext == NULL)
 	    {
-	      sPrint->SNext = TtaGetMemory (sizeof(StrSourcePrint));
+	      sPrint->SNext = (StrSourcePrint*)TtaGetMemory (sizeof(StrSourcePrint));
 	      sPrint = sPrint->SNext;
 	      sPrint->SNext = NULL;
 	      sPrint->STree = NULL;
@@ -947,6 +884,7 @@ PrintMethod method;
 	    }
 	  else
 	    sPrint = sPrint->SNext;
+	  }
       }
   if (found)
     {
@@ -955,7 +893,7 @@ PrintMethod method;
 	  for (i = 0; i< SIZEPRINT; i++)
 	    {
 	      sPrint->SNodes[i] = NULL;
-	      sPrint->SPrint[i] = '\0';
+	      sPrint->SPrint[i] = EOS;
 	    }
 	  sPrint->SPrintMethod = method;
 	  sPrint->STree = GetElemSourceTree ();
@@ -973,12 +911,7 @@ PrintMethod method;
 /*----------------------------------------------------------------------  
   RestNewRestruct
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 Restruct RestNewRestruct (ElementType typeDest)
-#else  /* __STDC__ */
-Restruct RestNewRestruct (typeDest)
-ElementType typeDest;
-#endif  /* __STDC__ */
 {
   Restruct restruct;
   int i;
@@ -989,8 +922,8 @@ ElementType typeDest;
   restruct->RNext = NULL;
   for (i = 0; i<SIZEPRINT; i++)
     {
-      restruct->RDestPrint[i] = '\0';
-      restruct->RDestNodes[i] = 
+      restruct->RDestPrint[i] = EOS;
+      restruct->RDestNodes[i] = NULL;
       restruct->RCoupledNodes[i] = NULL;
     }
   restruct->RDestTree = GetTypeTree (typeDest, FALSE);
@@ -1001,23 +934,17 @@ ElementType typeDest;
 /*----------------------------------------------------------------------  
   RestInitMatch : initialisation du contexte de transformation
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-boolean RestInitMatch (Element elemFirst, Element elemLast)
-#else  /* __STDC__ */
-boolean RestInitMatch (elemFirst, elemLast)
-Element elemFirst;
-Element elemLast;
-#endif  /* __STDC__ */
+ThotBool RestInitMatch (Element elemFirst, Element elemLast)
 {
   Element ptrElem;
   ElementType elType;
   Restruct lREff2,lREff;
   SourcePrint lSP, lSP2;
-  boolean ok = TRUE;
+  ThotBool ok = TRUE;
 
   if (RContext == NULL)
     {
-      RContext = TtaGetMemory(sizeof(StrRestContext));
+      RContext = (StrRestContext*)TtaGetMemory(sizeof(StrRestContext));
     }
   else
     { 
@@ -1085,22 +1012,15 @@ Element elemLast;
 /*----------------------------------------------------------------------  
   RestMatchElements
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-boolean RestMatchElements(Element elemFirst, Element elemLast, SSchema strSch, int typeNum)
-#else  /* __STDC__ */
-boolean RestMatchElements(elemFirst, elemLast, strSch, typeNum)
-Element elemFirst;
-Element elemLast;
-SSchema strSch;
-int typeNum;
-#endif  /* __STDC__ */
+ThotBool RestMatchElements(Element elemFirst, Element elemLast, SSchema strSch,
+			   int typeNum)
 {
   ElementType elType;
   Restruct restCour;
   Restruct pListRestruct;
   int i;
   SourcePrint sourcePrint;
-  boolean result;
+  ThotBool result;
 
 #ifdef DEBUG
   char msgbuf[100];
@@ -1121,7 +1041,7 @@ int typeNum;
   /* allocation et calcul de l'empreinte destination */
   restCour = RestNewRestruct(elType);
   strcpy (msgbuf,  TtaGetElementTypeName(restCour->RDestType));
-  printf("Destin : %s  -  %s\n",msgbuf,restCour->RDestPrint);  
+  printf("Destin : %s\n  -  %s\n",msgbuf,restCour->RDestPrint);  
   strcpy (msgbuf,  TtaGetElementTypeName(TtaGetElementType(elemFirst)));
   TtaFreeMemory (restCour);
 #endif
@@ -1131,13 +1051,11 @@ int typeNum;
   if (pListRestruct != NULL)
     while (pListRestruct->RNext != NULL)
       pListRestruct = pListRestruct->RNext;
-
-#ifdef DEBUG
-  strcpy (msgbuf,  TtaGetElementTypeName(elType));
-  printf("Source : %s\n", msgbuf);
-#endif
   /* boucle sur les methodes tant qu'on n'a pas trouve de solution */
-  for (i=0; /*restCour->RRelation == NONE &&*/ i < RContext->CNbPrintMethod; i++)
+  for (i = 0; 
+       (restCour == NULL || restCour->RRelation == NONE) && 
+	 i < RContext->CNbPrintMethod; 
+       i++)
     {
       restCour = RestNewRestruct(elType);
       /* instancie et calcule l'empreinte source */
@@ -1146,7 +1064,7 @@ int typeNum;
 	{
 	  restCour->RSrcPrint = sourcePrint;
 #ifdef DEBUG
-	  printf("Source : %s\n",  restCour->RSrcPrint->SPrint);
+	  printf("Src: %s\n",  restCour->RSrcPrint->SPrint);
 #endif
 	  /* compare les empreintes */
 	  RestMatchPrints (restCour);
@@ -1186,6 +1104,7 @@ int typeNum;
 	  printf ("Echec\n");
 #endif
 	  TtaFreeMemory (restCour);
+	  restCour = NULL;
 	}
     }
   return result; 
@@ -1196,22 +1115,15 @@ int typeNum;
   applique une transformation automatique pour changer sur place 
   le type d'un element 
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-boolean RestChangeType(Element elem, Document doc, int dstTypeNum, SSchema dstSch)
-#else  /* __STDC__ */
-boolean RestChangeType(elem, doc, dstTypeNum, dstSch)
-Element elem;
-Document doc;
-int dstTypeNum;
-SSchema dstSch;
-#endif  /* __STDC__ */
+ThotBool RestChangeType(Element elem, Document doc, int dstTypeNum, SSchema dstSch)
 {
-  boolean found = FALSE;
-  Restruct RestCour;
+  ThotBool    found = FALSE;
+  Restruct    cur;
   ElementType destType; 
 
   destType.ElTypeNum = dstTypeNum;
   destType.ElSSchema = dstSch;
+  cur = NULL;
   if (RContext->CNbOldElems != 1 || elem != RContext->COldElems[0])
     {
       /* l'instance source a change : regenere et recompare les empreintes */
@@ -1221,23 +1133,18 @@ SSchema dstSch;
   else
     {
       /* recherche dans les couplages existants */
-      RestCour = RContext->CListRestruct;
-      while (!found && RestCour != NULL)
+      cur = RContext->CListRestruct;
+      while (!found && cur != NULL)
 	{
-	  found = TtaSameTypes (RestCour->RDestType, destType);
+	  found = TtaSameTypes (cur->RDestType, destType);
 	  if (!found)
-	    RestCour = RestCour->RNext;
+	    cur = cur->RNext;
 	} 
     }
 
   if (found)
     /* applique la transformation */
-    return RestChangeOnPlace (doc, RestCour);
+    return RestChangeOnPlace (doc, cur);
   else
     return FALSE;
 }
-
-
-
-  
-
